# -*- coding: utf-8 -*-
from __future__ import absolute_import
"""
admin module for jmb.newsletter2 app

"""
# -*- coding: utf-8 -*-
import os
from copy import deepcopy
from HTMLParser import HTMLParseError

from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.db.models import Q
from django.conf.urls import patterns, url
from django.contrib import admin, messages
from django.http import (
    HttpResponseRedirect, HttpResponse, Http404,
    HttpResponseBadRequest, HttpResponseForbidden, HttpResponseNotAllowed, HttpResponseServerError
)
from django.core.urlresolvers import reverse
from django.forms import CharField
from django.shortcuts import get_object_or_404, render_to_response
from django.template.context import RequestContext
from django.template.defaultfilters import title, escapejs, force_escape
from django.utils.encoding import force_unicode
from django.utils.translation import ugettext as _

#from cms.forms.widgets import PluginEditor
from jmb.newsletter2.forms.widgets import PluginEditor

from cms.models import CMSPlugin, Placeholder
from cms.plugin_pool import plugin_pool
from cms.templatetags.cms_admin import admin_static_url
from cms.utils import (
    get_template_from_request,
    placeholder as placeholder_utils,
    get_language_from_request,
    cms_static_url
)

from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.newsletter2.models import (Newsletter, NewsletterTemplate, MailingList)
from jmb.newsletter2.admin.forms import NewsletterAddForm
#from jmb.newsletter2.forms.widgets import PluginEditor
from jmb.newsletter2.utils import plugins, newsletter2_static_url
#from jmb.newsletter2.admin.views import newsletter2_statistics
#from jmb.mailer.sender import Mailer, EmailObject # FIXME: creare le due classi in sender2.py

PAGE_ADMIN_INLINES = []
create_on_success = lambda x: x
#newsletter_static_url = os.path.join(settings.STATIC_URL, "newsletter2")

class NewsletterAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            'all': [newsletter2_static_url(path) for path in (
                'css/rte.css',
                'css/pages.css',
                'css/change_form.css',
                'css/jquery.dialog.css',
                'css/admin_newsletter.css'
                )]
        }
        js = [
            newsletter2_static_url(path) for path in [
                'js/plugins/admincompat.js',
                'js/libs/jquery.query.js',
                'js/libs/jquery.ui.core.js',
                'js/libs/jquery.ui.dialog.js',
            ]
        ]

    change_list_template = "admin/newsletter2/newsletter/change_list.html"
    change_form_template = "admin/newsletter2/newsletter/change_form.html"
    add_general_fields = ['title', 'slug', 'template', 'status']
    edit_fieldsets = [
        (_("Ordering"), {
            'fields' : ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'title', 'slug', 'sending_date',),
            'classes': ('general',),
        }),
        (_('Content web'), {
            'fields': ('content_web',),
            'classes':('collapse',),
        }),
        (_('Basic Settings'), {
            'fields': ('template',),
            'classes': ('low',),
           # 'description': _('Note: This page reloads if you change the selection. Save it first.'),
        }),
    ]
    add_fieldsets = [
        (None, {
            'fields': add_general_fields,
            'classes': ('general',),
        }),
        (_('Content web'), {
            'fields': ('content_web',),
            'classes':('collapse',),
        }),
    ]
    # TODO: FIXME - da sistemare quando il modello è completo con tutti i campi
    #    list_display = (
    #        'status', 'title', 'mailing_list', 'historic_link', 'statistics_link',
    #        'sending_date','get_edit_icon', 'get_delete_icon'
    #    )
    list_display = (
        'status', 'sending_date','title', 'template','historic_link', 'statistics_link',
        'sending_date', 'get_edit_icon', 'get_delete_icon'
    )
    inlines = PAGE_ADMIN_INLINES


    # ACTIONS
    def get_actions(self, request):
        actions = super(NewsletterAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        if not request.user.has_perm('newsletter.can_change_status'):
            del actions['make_ready_to_send']
            del actions['make_cancel_sending']
            del actions['reset_status_values']
        return actions

    def send_newsletter_test(self, request, queryset): # da eliminare
        """Send newsletter in test"""
        for newsletter in queryset:
            if newsletter.mailing_list.has_test_contact:
                mailer = Mailer(newsletter, test=True)
                try:
                    mailer.run()
                except HTMLParseError:
                    messages.error(request, _('Unable send newsletter, due to errors within HTML.'))
                    continue
                self.message_user(request, _('%s succesfully sent.') % newsletter.title)
            else:
                messages.warning(request, _('No test contacts assigned for %s.') % newsletter)
    send_newsletter_test.short_description = _('Send test newsletter')

    def make_ready_to_send(self, request, queryset):
        """Make newsletter ready to send"""
        queryset = queryset.filter(status=settings.NEWSLETTER2_STATUS_DRAFT)
        for newsletter in queryset:
            newsletter.status = settings.NEWSLETTER2_STATUS_WAITING
            newsletter.save()
        self.message_user(request, _('%s newletters are ready to send') % queryset.count())
    make_ready_to_send.short_description = _('Make ready to send')

    def make_cancel_sending(self, request, queryset):
        """Cancel the sending of newsletters"""
        queryset = queryset.filter(
            Q(status=settings.NEWSLETTER2_STATUS_WAITING) |
            Q(status=settings.NEWSLETTER2_STATUS_SENDING)
        )
        for newsletter in queryset:
            newsletter.status = settings.NEWSLETTER2_STATUS_CANCELED
            newsletter.save()
        self.message_user(request, _('%s newletters are cancelled') % queryset.count())
    make_cancel_sending.short_description = _('Cancel the sending')

    def reset_status_values(self, request, queryset):
        """
        Reset all Newsletters status values
        """
        count = queryset.count()
        queryset = queryset.filter(status=settings.NEWSLETTER2_STATUS_DRAFT)
        filtered_count = queryset.count()
        if count != filtered_count:
            messages.error(request, _("You can reset only DRAFT newsletters"))
        newsletter_content_type=ContentType.objects.get_for_model(Newsletter)
        for newsletter in queryset:
            emails = EmailObject.objects.filter(
                content_type=newsletter_content_type,
                object_id=newsletter.id
            )
            messages.info(request, "%s: reset status values" % newsletter.title)
            emails.delete()
    reset_status_values.short_description = _("Reset status values")

    ## LIST DISPLAY METHODS
    # FIXME: da rivedere
    def historic_link(self, newsletter):
        """
        Display link for historic

        """
        newsletter_type = ContentType.objects.get_for_model(newsletter)
        if newsletter.get_email_mailing_status().count() and newsletter.slug:
            return u'<a href="%(href)s?' \
                   u'email_object__content_type__id=%(n_type_id)s&' \
                   u'email_object__object_id=%(id)s">%(title)s</a>' % {
                'href':         reverse('admin:mailer_mailingstatus_changelist'),
                'title':        _('View historic'),
                'n_type_id':    newsletter_type.id,
                'id':           newsletter.id
            }
        return _('Not available')
    historic_link.allow_tags = True
    historic_link.short_description = _('Historic')

    # FIXME: da rivedere
    def statistics_link(self, newsletter):
        """Display link for statistics"""
        newsletter_type = ContentType.objects.get_for_model(newsletter)
        if newsletter.get_email_mailing_status().count() and newsletter.slug:
            return u"""
                <a href="%(href)s">
                    <img
                        src="%(url)sadmin/images/statistics.png"
                        alt="%(title)s" title="%(title)s"
                    />
                </a>""" % {
                'href':     reverse(
                    'admin:newsletter2_newsletter_statistics',
                    args=[newsletter.slug]
                ),
                'url':settings.MEDIA_URL,
                'title':    _('View statistics')
            }
        return _('Not available')
    statistics_link.allow_tags = True
    statistics_link.short_description = _('Stats')

    def get_urls(self):
        """Get the admin urls
        """
        info = "%s_%s" % (self.model._meta.app_label, self.model._meta.module_name)
        pat = lambda regex, fn: url(regex, self.admin_site.admin_view(fn), name='%s_%s' % (info, fn.__name__))

        url_patterns = patterns('',
            # TODO: FIXME
            pat(r'add-plugin/$', self.add_plugin),
            pat(r'edit-plugin/([0-9]+)/$', self.edit_plugin),
            pat(r'remove-plugin/$', self.remove_plugin),
            pat(r'move-plugin/$', self.move_plugin),
            pat(r'^(?P<object_id>\d+)/change_template/$', self.change_template), # copy dialog
            pat(r'^(?P<object_id>\d+)/view/', self.view_newsletter),
            # TODO: FIXME
#            pat(r'^statistics/(?P<slug>[-\w]+)/$', newsletter2_statistics),
        )

        url_patterns = url_patterns + super(NewsletterAdmin, self).get_urls()
        return url_patterns

# TODO: FIXME
#    @create_on_success
#    def change_template(self, request, object_id):
#    DA VEDERE BENE - UTILIZZA ATTUALMENTE QUELLA DEL CMS

    def get_fieldsets(self, request, obj=None):
        """
        Add fieldsets of placeholders to the list of already existing
        fieldsets.
        """

        # FIXME: mod successiva -> sia in add che in edit utilizza gli edit_fieldsets
        given_fieldsets = deepcopy(self.edit_fieldsets)
        if obj: # edit
            #given_fieldsets = deepcopy(self.edit_fieldsets)
            placeholders_template = obj.template.file.path
            for placeholder_name in self.get_fieldset_placeholders(placeholders_template):
                name = placeholder_utils.get_placeholder_conf("name", placeholder_name, obj.template, placeholder_name)
                name = _(name)
                given_fieldsets += [(title(name), {'fields':[placeholder_name], 'classes':['plugin-holder']})]
        #else: # add
        #    given_fieldsets = deepcopy(self.add_fieldsets)

        return given_fieldsets

    def get_fieldset_placeholders(self, template):
        return plugins.get_placeholders(template)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        try:
            obj = self.model.objects.get(pk=object_id)
        except:
            raise Http404
        extra_context = extra_context or {}
        extra_context.update({
            'language': 'it',
        })
        return super(NewsletterAdmin, self).change_view(request, object_id, form_url, extra_context)

    def get_form(self, request, obj=None, **kwargs):
        """
        Get NewsletterForm for the Newsletter model and modify its fields depending on
        the request.
        """

        language = get_language_from_request(request, obj)
        language = 'it'

        if obj:
#            self.inlines = PAGE_ADMIN_INLINES
            form = super(NewsletterAdmin, self).get_form(request, obj, **kwargs)
        else:
            self.inlines = []
            form = NewsletterAddForm

        if obj:
            templates = NewsletterTemplate.objects.all()
            if templates:
                selected_template = obj.template
                form.base_fields['template'].queryset = templates
                form.base_fields['template'].initial = selected_template

            placeholders = self.get_fieldset_placeholders(selected_template.file.path)
            for placeholder_name in placeholders:
                show_copy = False
                copy_languages = {}
                placeholder, created = obj.placeholders.get_or_create(slot=placeholder_name)
                installed_plugins = plugin_pool.get_all_plugins(placeholder_name, obj)
                plugin_list = CMSPlugin.objects.filter(language=language, placeholder=placeholder, parent=None).order_by('position')
                other_plugins = CMSPlugin.objects.filter(placeholder=placeholder, parent=None).exclude(language=language)
                dict_cms_languages = dict(settings.CMS_LANGUAGES)
                for plugin in other_plugins:
                    if (not plugin.language in copy_languages) and (plugin.language in dict_cms_languages):
                        copy_languages[plugin.language] = dict_cms_languages[plugin.language]

                language = get_language_from_request(request, obj)
                if copy_languages and len(settings.CMS_LANGUAGES) > 1:
                    show_copy = True
                widget = PluginEditor(attrs={
                    'installed': installed_plugins,
                    'list': plugin_list,
                    'copy_languages': copy_languages.items(),
                    'show_copy': show_copy,
                    'language': language,
                    'placeholder': placeholder
                })
                form.base_fields[placeholder.slot] = CharField(widget=widget, required=False)
        else:
            for name in ['slug','title']:
                form.base_fields[name].initial = u''
            newsletter_template = ""
            newsletter_templates = NewsletterTemplate.objects.all()
            if newsletter_templates:
                newsletter_template = newsletter_templates[0]
            form.base_fields['template'].initial = newsletter_template
        form.base_fields['sending_date'].required = False
        return form


    def view_newsletter(self, request, object_id):
        try:
            newsletter = Newsletter.objects.get(id=object_id)
        except:
            raise Http404

        template_name = newsletter.template.file.path
        context = RequestContext(request)
        context['newsletter'] = newsletter
        return render_to_response(template_name, context)


    def add_plugin(self, request):
        '''
        Could be either a page or a parent - if it's a parent we get the page via parent.
        '''
        if request.method != "POST":
            raise Http404
        plugin_type = request.POST.get('plugin_type', None)
        placeholder_id = request.POST.get('placeholder', None)
        if placeholder_id:
            placeholder = get_object_or_404(Placeholder, pk=placeholder_id)
            newsletters = placeholder.newsletters.all()
            if newsletters:
                newsletter = newsletters[0]
        else:
            placeholder = None
            newsletter = None
        # newsletter add-plugin
        if newsletter:
            language = request.POST.get('language', None) or get_language_from_request(request)
            position = CMSPlugin.objects.filter(language=language, placeholder=placeholder).count()
            limits = placeholder_utils.get_placeholder_conf("limits", placeholder.slot, newsletter.get_template())
            if limits:
                global_limit = limits.get("global")
                type_limit = limits.get(plugin_type)
                if global_limit and position >= global_limit:
                    return HttpResponseBadRequest("This placeholder already has the maximum number of plugins")
                elif type_limit:
                    type_count = CMSPlugin.objects.filter(language=language, placeholder=placeholder, plugin_type=plugin_type).count()
                    if type_count >= type_limit:
                        plugin_name = unicode(plugin_pool.get_plugin(plugin_type).name)
                        return HttpResponseBadRequest("This placeholder already has the maximum number allowed of %s plugins." % plugin_name)
        # in-plugin add-plugin
        else:
            # do NOT allow non-newsletter placeholders to use this method, they
            # should use their respective admin!
            raise Http404

        # TODO: FIXME
        language = settings.NEWSLETTER2_LANGUAGES[0][0]

        plugin = CMSPlugin(language=language, plugin_type=plugin_type, position=position, placeholder=placeholder)

        plugin.save()

        return HttpResponse(str(plugin.pk))

    @create_on_success
    def edit_plugin(self, request, plugin_id):
        plugin_id = int(plugin_id)
        # history view with reversion
        pre_edit = request.path.split("/edit-plugin/")[0]

        cms_plugin = CMSPlugin.objects.get(id=plugin_id)

        inst, plugin_admin = cms_plugin.get_plugin_instance(self.admin_site)
        instance = None
        if inst:
            instance = cms_plugin

        plugin_admin.cms_plugin_instance = cms_plugin

        try:
            plugin_admin.placeholder = cms_plugin.placeholder
        except Placeholder.DoesNotExist:
            pass

        if request.method == "POST":
            # set the continue flag, otherwise will plugin_admin make redirect to list
            # view, which actually doesn't exists
            request.POST['_continue'] = True

        if request.POST.get("_cancel", False):
            # cancel button was clicked
            context = {
                'CMS_MEDIA_URL': settings.CMS_MEDIA_URL,
                'plugin': cms_plugin,
                'is_popup': True,
                'name': unicode(cms_plugin),
                "type": cms_plugin.get_plugin_name(),
                'plugin_id': plugin_id,
                'icon': force_escape(escapejs(cms_plugin.get_instance_icon_src())),
                'alt': force_escape(escapejs(cms_plugin.get_instance_icon_alt())),
                'cancel': True,
                }
            instance = cms_plugin.get_plugin_instance()[0]
            if not instance:
                # cancelled before any content was added to plugin
                cms_plugin.delete()
                context.update({
                    "deleted":True,
                })
            return render_to_response('admin/newsletter2/newsletter/plugin_forms_ok.html', context, RequestContext(request))

        if not instance:
            # instance doesn't exist, call add view
            response = plugin_admin.add_view(request)

        else:
            # already saved before, call change view
            # we actually have the instance here, but since i won't override
            # change_view method, is better if it will be loaded again, so
            # just pass id to plugin_admin
            response = plugin_admin.change_view(request, str(plugin_id))
        if request.method == "POST" and plugin_admin.object_successfully_changed:
            saved_object = plugin_admin.saved_object
            context = {
                'CMS_MEDIA_URL': settings.CMS_MEDIA_URL,
                'plugin': saved_object,
                'is_popup': True,
                'name': unicode(saved_object),
                "type": saved_object.get_plugin_name(),
                'plugin_id': plugin_id,
                'icon': force_escape(saved_object.get_instance_icon_src()),
                'alt': force_escape(saved_object.get_instance_icon_alt()),
            }
            return render_to_response(
                'admin/newsletter2/newsletter/plugin_forms_ok.html', context, RequestContext(request)
            )

        return response

    @create_on_success
    def move_plugin(self, request):
        if request.method != "POST":
            return HttpResponse(str("error"))
        pos = 0
        newsletter = None
        success = False
        if 'plugin_id' in request.POST:
            plugin = CMSPlugin.objects.get(pk=int(request.POST['plugin_id']))
            if plugin.placeholder:
                newsletters = plugin.placeholder.newsletters.all()
                if newsletters:
                    newsletter = newsletters[0]
                else:
                    return HttpResponse(str("error"))

            placeholder_to = request.POST.get('placeholder', None)
            #placeholders = self.get_fieldset_placeholders(newsletter.get_template())
            placeholders = self.get_fieldset_placeholders(newsletter.template.file.path)
            if not placeholder_to in placeholders:
                return HttpResponse(str("error"))
            placeholder = newsletter.placeholders.get(slot=placeholder_to)
            plugin.placeholder = placeholder
            # plugin positions are 0 based, so just using count here should give us 'last_position + 1'
            position = CMSPlugin.objects.filter(placeholder=placeholder).count()
            plugin.position = position
            plugin.save()
            success = True
        if 'ids' in request.POST:
            for plugin_id in request.POST['ids'].split("_"):
                plugin = CMSPlugin.objects.select_related('placeholder').get(pk=plugin_id)
                if plugin.placeholder:
                    newsletters = plugin.placeholder.newsletters.all()
                    if newsletters:
                        newsletter = newsletters[0]
                if not newsletter:
                    raise Http404

                if plugin.position != pos:
                    plugin.position = pos
                    plugin.save()
                pos += 1
            success = True
        if not success:
            return HttpResponse(str("error"))

        return HttpResponse(str("ok"))

    @create_on_success
    def remove_plugin(self, request):
        if request.method != "POST":
            raise Http404
        plugin_id = request.POST['plugin_id']
        plugin = get_object_or_404(CMSPlugin.objects.select_related('placeholder'), pk=plugin_id)
        plugin.delete_with_public()
        plugin_name = unicode(plugin_pool.get_plugin(plugin.plugin_type).name)
        comment = _(u"%(plugin_name)s plugin at position %(position)s in %(placeholder)s was deleted.") % {
            'plugin_name': plugin_name,
            'position': plugin.position,
            'placeholder': plugin.placeholder,
            }
        return HttpResponse("%s,%s" % (plugin_id, comment))

    @create_on_success
    def change_template(self, request, object_id):
        newsletter = get_object_or_404(Newsletter, pk=object_id)
        #if page.has_change_permission(request):
        to_template = request.POST.get("template", None)
        #    if to_template:
        #        newsletter.template = to_template
        #        page.save()
        #        if "reversion" in settings.INSTALLED_APPS:
        #            helpers.make_revision_with_plugins(page)
        #        return HttpResponse(str("ok"))
        #    else:
        #        return HttpResponseBadRequest("template not valid")
        #else:
        return HttpResponseForbidden(_("You have no permission to change the template"))


class MailingListAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter2/css/admin_newsletter.css",)}

    date_hierarchy = 'date_create'
    list_display = (
        'get_status_icon','date_create', 'name',
        'subscriptions', 'test_subscriptions', 'unsubscriptions',
        'get_edit_icon','get_delete_icon'
        )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields' : ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status','name', 'slug','description',)
        }),
        (_("SEO settings"), {
            'fields' : ('page_title','meta_description','meta_keywords'),
            'classes': ('collapse',)
        }),
    )
    prepopulated_fields = {'slug': ('name',)}
    advanced_search_fields = (('name',),)

class NewsletterTemplateAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter2/css/admin_newsletter.css",)}

    list_display = (
        'get_status_icon','date_create', 'name',
        'get_edit_icon','get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields' : ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('name', 'file',)
        }),
    )

admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(NewsletterTemplate, NewsletterTemplateAdmin)
admin.site.register(MailingList, MailingListAdmin)