# -*- coding: utf-8 -*-
"""
plugin models definition for jmb.newsletter2 app. To be used only with
jmb.newsletter2 app (not with cms)
"""

from django.db import models
from django.utils.translation import ugettext as _
from django.conf import settings

from cms.models.pluginmodel import CMSPlugin

from filer.fields.image import FilerImageField

from jmb.core.db.cmsmodels import StatusModel, WrapModel
from jmb.news.models import News
from acomea.fondi.models import Fund, Diary

###################################
## CMS MODELS FOR NEWSLETTER      #
###################################


class Newsletter2TemplateChoiceSettings(CMSPlugin, StatusModel):
    """
    template choiche plugin model
    """
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.NEWSLETTER2_CMSPLUGIN_TEMPLATECHOICES_TEMPLATES,
        default="contact/cms_plugins/template_choice1.html",
        help_text=_("Choiche a template to include in the HTML")
    )

    def __unicode__(self):
        _unicode = "%s - %s" % (self._meta.verbose_name, self.template_name.split('/').pop())
        if self.status:
            return "On - %s" % _unicode
        else:
            return "Off - %s" % _unicode

    class Meta:
        verbose_name = _("Template Choiche")
        app_label = "newsletter2"


class Newsletter2NewsDetailSettings(CMSPlugin, StatusModel):
    """
    news detail plugin model
    """
    template_name = models.CharField(
        _("Template name"),
        max_length=150,
        choices=settings.NEWSLETTER2_CMSPLUGIN_NEWSDETAIL_TEMPLATES,
        default="contact/cms_plugins/news_detail.html"
    )
    news = models.ForeignKey(
        News,
        verbose_name=_("News")
    )

    def __unicode__(self):
        _unicode = "%s - %s" % (
            self._meta.verbose_name, self.news.title
        )
        if self.status:
            return "On - %s" % _unicode
        else:
            return "Off - %s" % _unicode

    class Meta:
        verbose_name = _("News detail")
        app_label = "newsletter2"


class Newsletter2DiaryDetailSettings(CMSPlugin, StatusModel):
    """
    diary detail plugin model
    """
    template_name = models.CharField(
        _("Template name"),
        max_length=150,
        choices=settings.NEWSLETTER2_CMSPLUGIN_DIARYDETAIL_TEMPLATES,
        default="contact/cms_plugins/diary_detail.html"
    )
    fund = models.ForeignKey(
        Fund,
        verbose_name=_("Fund"),
        null=True, blank=True
    )
    diary = models.ForeignKey(
        Diary,
        verbose_name=_("Diary")
    )
    image = FilerImageField(
        null=True, blank=True,
        verbose_name=_("Diary image"),
        help_text=_("Optional image field for diary")
    )

    def __unicode__(self):
        _unicode = "%s - %s" % (
            self._meta.verbose_name, self.diary.title
        )
        if self.status:
            return "On - %s" % _unicode
        else:
            return "Off - %s" % _unicode

    class Meta:
        verbose_name = _("Diary detail")
        app_label = "newsletter2"

