# -*- coding: utf-8 -*-
from django import forms
from django.core.paginator import Paginator
from django.utils.translation import ugettext as _
from django.conf import settings
from django.core.urlresolvers import resolve

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms.plugins.snippet.cms_plugins import SnippetPlugin
from cmsplugin_filer_image.cms_plugins import FilerImagePlugin

from jmb.cms.custom_text.cms.plugins import CustomTextPlugin
from jmb.cms.script.cms.plugins import ScriptPlugin
from jmb.newsletter2.cms.models import (
    Newsletter2DiaryDetailSettings,
    Newsletter2TemplateChoiceSettings,
    Newsletter2NewsDetailSettings
)

#FIXME: controllare l'utilizzo al di fuori del progetto www.acomea.it
from web.utils.generic import get_acomea_host, get_acomea_tcp_protocol

###################################
## CMS PLUGINS                    #
###################################


class Newsletter2TemplateChoicePlugin(CMSPluginBase):
    model = Newsletter2TemplateChoiceSettings
    name = _("Template choice")
    module = _("Newsletter2")
    admin_preview = False

    def has_add_permission(self, request, *args, **kwargs):
        return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        return super(CMSPluginBase, self).has_change_permission(request, obj)

    def render(self, context, instance, placeholder):
        self.render_template = settings.NEWSLETTER2_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        self.render_template = instance.template_name

        context.update({
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class Newsletter2DiaryDetailPlugin(CMSPluginBase):
    model = Newsletter2DiaryDetailSettings
    name = _("Diary detail")
    module = _("Newsletter2")
    admin_preview = False


    def has_add_permission(self, request, *args, **kwargs):
        return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        return super(CMSPluginBase, self).has_change_permission(request, obj)


    def render(self, context, instance, placeholder):
        self.render_template = settings.NEWSLETTER2_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        request = context['request']
        self.render_template = instance.template_name
        host = "%s://%s" % (get_acomea_tcp_protocol(request), get_acomea_host())

        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'object':instance.diary,
            'host':host
        })
        return context


class Newsletter2NewsDetailPlugin(CMSPluginBase):
    model = Newsletter2NewsDetailSettings
    name = _("News detail")
    module = _("Newsletter2")
    admin_preview = False

    def has_add_permission(self, request, *args, **kwargs):
        return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        return super(CMSPluginBase, self).has_change_permission(request, obj)

    def render(self, context, instance, placeholder):
        self.render_template = settings.NEWSLETTER2_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        request = context['request']
        self.render_template = instance.template_name
        host = "%s://%s" % (get_acomea_tcp_protocol(request), get_acomea_host())
        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'object':instance.news,
            'host':host

        })
        return context


## FIXME: codice preso da acomea.profilo.utils -> da spostare in helper comune perche' riutilizzato in piu' parti
#def get_acomea_host():
#    try:
#        return settings.PROFILO_HOST
#    except AttributeError:
#        pass
#    return "127.0.0.1:8000"
#
#
## FIXME: codice preso da acomea.profilo.utils -> da spostare in helper comune perche' riutilizzato in piu' parti
#def get_acomea_tcp_protocol(request):
#    try:
#        return settings.PROFILO_TCP_PROTOCOL
#    except AttributeError:
#        pass
#    return "https" if request.is_secure() else "http"

"""
fix per permessi:
risolvendo la request.path, controlla se siamo in cms o newsletter2:
- nel primo caso usa i metodi 'has_add_permission' e 'has_change_permission' di CMSPluginBase;
- nel secondo caso usa quelli di ModelAdmin;

necessario per le seguenti app che forniscono plugins in comune tra cms e newsletter2:
snippet, custom_text, cmsplugin_filer_image, script
"""

# snippet plugin
plugin_pool.unregister_plugin(SnippetPlugin)
class SnippetPlugin(SnippetPlugin):

    def has_add_permission(self, request, *args, **kwargs):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)
        else:
            return super(SnippetPlugin, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_change_permission(request, obj)
        else:
            return super(SnippetPlugin, self).has_change_permission(request, obj)
plugin_pool.register_plugin(SnippetPlugin)

# filer image plugin
plugin_pool.unregister_plugin(FilerImagePlugin)
class FilerImagePlugin(FilerImagePlugin):

    def has_add_permission(self, request, *args, **kwargs):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)
        else:
            return super(FilerImagePlugin, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_change_permission(request, obj)
        else:
            return super(FilerImagePlugin, self).has_change_permission(request, obj)
plugin_pool.register_plugin(FilerImagePlugin)


# custom text plugin
plugin_pool.unregister_plugin(CustomTextPlugin)
class CustomTextPlugin(CustomTextPlugin):

    def has_add_permission(self, request, *args, **kwargs):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)
        else:
            return super(CustomTextPlugin, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_change_permission(request, obj)
        else:
            return super(CustomTextPlugin, self).has_change_permission(request, obj)
plugin_pool.register_plugin(CustomTextPlugin)


# script plugin
plugin_pool.unregister_plugin(ScriptPlugin)
class ScriptPlugin(ScriptPlugin):

    def has_add_permission(self, request, *args, **kwargs):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_add_permission(request, *args, **kwargs)
        else:
            return super(ScriptPlugin, self).has_add_permission(request, *args, **kwargs)

    def has_change_permission(self, request, obj=None):
        resolver_match = resolve(request.path)
        try:
            view_name = resolver_match.view_name.split(':')[1]
        except IndexError:
            view_name = None
        if view_name and view_name.startswith('newsletter2'):
            return super(CMSPluginBase, self).has_change_permission(request, obj)
        else:
            return super(ScriptPlugin, self).has_change_permission(request, obj)
plugin_pool.register_plugin(ScriptPlugin)


plugin_pool.register_plugin(Newsletter2TemplateChoicePlugin)
plugin_pool.register_plugin(Newsletter2DiaryDetailPlugin)
plugin_pool.register_plugin(Newsletter2NewsDetailPlugin)

