from __future__ import absolute_import
# coding: utf-8

import datetime

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models.placeholdermodel import Placeholder
from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.core.db.fields import HTMLFieldTinyMCE

## TODO: FIXME - deve prendere da db
#template_choices = [(x, _(y)) for x, y in settings.CMS_TEMPLATES]

class Newsletter(UserModel, DateModel, OrderedModel, SEOModel):
    @classmethod
    def get_years(cls, desc=True):
        output = []
        newsletter_qs = Newsletter.objects.filter(status=settings.NEWSLETTER2_STATUS_SENT)
        for d in newsletter_qs.dates('sending_date','year', order='DESC' if desc else 'ASC'):
            output.append((str(d.year),str(d.year)))
        return output

    @classmethod
    def get_months(cls):
        return [
            ("1", _("January")),
            ("2", _("February")),
            ("3", _("March")),
            ("4", _("April")),
            ("5", _("May")),
            ("6", _("June")),
            ("7", _("July")),
            ("8", _("August")),
            ("9", _("September")),
            ("10", _("October")),
            ("11", _("November")),
            ("12", _("December"))
        ]

    def clone(self):
        """
        clone Newsletter
        """
        today = datetime.today()
        new_obj = Newsletter()
        new_obj.title = '%s_%s' % (self.title, today.strftime('%d-%m-%Y'))
        new_obj.slug = new_obj.title
        # FIXME: i seguenti campi non sono presenti nel model, come non lo erano nel
        # FIXME: model newsletter precedente
        new_obj.server = self.server
        new_obj.header_sender = self.header_sender
        new_obj.header_reply= self.header_reply

        new_obj.save()

        return new_obj

    @models.permalink
    def ge_absolute_url(self):
        return ('newsletter2_newsletter_preview', (self.slug))

    @models.permalink
    def get_historic_url(self):
        return ('newsletter2_newsletter_historic', (self.slug,))

    @models.permalink
    def get_statistics_url(self):
        return ('newsletter2_newsletter_statistics', (self.slug,))




    status = models.IntegerField(
        _('Newsletter status'),
        choices= settings.NEWSLETTER2_STATUS_CHOICES,
        default=settings.NEWSLETTER2_STATUS_DRAFT
    )
    title = models.CharField(
        _("Title"),
        max_length=255,
        help_text=_(
            "You can use the '{{ UNIQUE_KEY }}' variable for unique identifier within the newsletter's title."
        )
    )
    slug = models.SlugField(
        _("Slug"),
        help_text=_('Used for displaying the newsletter on the site.'),
        unique=True
    )
    content_web = HTMLFieldTinyMCE(
        _('Content WEB'),
        help_text=_('Write web HTML code.'),
    )
    sending_date = models.DateTimeField(
        null=True, blank=True,
    )
    template = models.ForeignKey(
        "NewsletterTemplate",
        help_text=_('The template used to render the content.')
    )
    placeholders = models.ManyToManyField(
        Placeholder,
        editable=False,
        related_name='newsletters'
    )

    class Meta(object):
        verbose_name = _('Newsletter')
        verbose_name_plural = _('Newsletters')

    def __unicode__(self):
        return self.title

    def get_template(self):
        """
        get the template of this page if defined or if closer parent if
        defined or DEFAULT_PAGE_TEMPLATE otherwise
        """
#        if hasattr(self, '_template_cache'):
#            return self._template_cache
        template = self.template
        self._template_cache = template
        return template

    def get_template_name(self):
        """
        get the textual name (2nd parameter in settings.CMS_TEMPLATES)
        of the template of this page or of the nearest
        ancestor. failing to find that, return the name of the default template.
        """
        return self.template.name

    def reload(self):
        """
        Reload a newsletter from the database
        """
        return Newsletter.objects.get(pk=self.pk)

    def get_object_queryset(self):
        """Returns smart queryset depending on object type - draft / public
        """
        return self.__class__.objects

    def rescan_placeholders(self):
        """
        Rescan and if necessary create placeholders in the current template.
        """
        # inline import to prevent circular imports
        from jmb.newsletter2.utils.plugins import get_placeholders
        # TODO: FIXME
        placeholders = get_placeholders(self.get_template())
        found = {}
        for placeholder in self.placeholders.all():
            if placeholder.slot in placeholders:
                found[placeholder.slot] = placeholder
        for placeholder_name in placeholders:
            if not placeholder_name in found:
                placeholder = Placeholder.objects.create(slot=placeholder_name)
                self.placeholders.add(placeholder)
                found[placeholder_name] = placeholder

    def get_default_title(self):
        """
        Used for multilanguage templates
        """
        return self.title

    def get_default_template_path(self):
        """
        Used for multilanguage templates
        """
        return self.template.file.path

class NewsletterTemplate(UserModel, DateModel, OrderedModel):
    name = models.CharField(
        _("Name"),
        unique=True,
        max_length=255,
    )
    file = models.FileField(
        upload_to=settings.NEWSLETTER2_TEMPLATE_PATH,
        verbose_name=_("Template"),
    )

    class Meta:
        ordering = ('name',)
        verbose_name = _('Template')
        verbose_name_plural = _('Templates')

    def __unicode__(self):
        return "%s" % self.name


class MailingList(StatusModel, UserModel, DateModel, OrderedModel, SEOModel):
    """Mailing list"""
    name = models.CharField(
        _("Name"),
        max_length=255
    )
    slug = models.SlugField(
        _("Slug"),
        unique=True,
        max_length=100,
        default=datetime.datetime.now().strftime("%d-%m-%Y__%H%M")
    )
    description = models.TextField(
        _("Description"),
        blank=True, null=True
    )
    def get_subscribers(self):
        """ return Subscribers Contact queryset """
        return self.contacts.filter(
            date_unsubscription__isnull=True,
            status=1
    )
    subscribers = property(get_subscribers)

    def get_test_subscribers(self):
        """ return Test Subscribers Contact queryset """
        return self.subscribers.filter(
            is_test_contact=True
        )
    test_subscribers = property(get_test_subscribers)

    def get_unsubscribers(self):
        """ return Unsubscribers Contact queryset """
        return self.contacts.filter(
            date_unsubscription__isnull=False,
        )
    unsubscribers = property(get_unsubscribers)

    def get_subscriptions(self):
        """
        return number of subscribtions
        """
        return self.subscribers.count()
    subscriptions = property(get_subscriptions)

    def get_test_subscriptions(self):
        """
        return number of test subscribtions
        """
        return self.test_subscribers.count()
    test_subscriptions = property(get_test_subscriptions)

    def get_unsubscriptions(self):
        """
        return number of unsubscribtions
        """
        return self.unsubscribers.count()
    unsubscriptions = property(get_unsubscriptions)

    def has_test_contact(self):
        """ return True or False if Test Contacts are present """
        if self.test_subscribers.count() > 0:
            return True
        else:
            return False
    has_test_contact = property(has_test_contact)

    #def add_subscriber(self, object):
    #    """
    #    to add subscriber to this mailinglist
    #    subscriber is contact object
    #    return a tuple with True and contact object if success
    #    else return a tuple with False and error message
    #    """
    #    if not email_re.match(object.email):
    #        return (False, _("Not valid email address"))
    #
    #    try:
    #        contact = Subscriber.objects.get(
    #            mailinglist=self, email__iexact=object.email
    #        )
    #        return (False, _("Email already exists"))
    #
    #    except:
    #        contact = Subscriber(
    #            mailinglist=self,
    #            content_object=object,
    #            email=object.email
    #        )
    #        contact.save()
    #        return (True, contact)

    def clone(self):
        """
        clone mailinglist without contacts
        """
        today = datetime.today()
        new_obj = MailingList()
        new_obj.name = '%s_%s' % (self.name, today.strftime('%d-%m-%Y'))
        new_obj.description = self.description
        new_obj.save()

        return new_obj

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('Mailing list')
        verbose_name_plural = _('Mailing lists')





if settings.CMS_ENABLED:
    import jmb.newsletter2.cms.plugins
#     import acomea.newsletter.cms.app
